# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import shlex
from ..common.errors import CliParsingError, ProhibitedOperatorsError


excluded = frozenset(
    {
        '&&',
        '||',
        '=',
        '*=',
        '/=',
        '%=',
        '+=',
        '-=',
        '<<=',
        '>>=',
        '&=',
        '^=',
        '|=',
    }
)


def split_cli_command(cli_command: str) -> list[str]:
    """Split the given CLI command into multiple tokens."""
    try:
        tokens = shlex.split(cli_command)
    except ValueError as e:
        raise CliParsingError(e) from e
    prohibited_tokens = [token for token in tokens if token in excluded]
    if prohibited_tokens:
        raise ProhibitedOperatorsError(prohibited_tokens)
    if not tokens:
        raise CliParsingError('The provided CLI command is empty')
    command = tokens[0]
    if command != 'aws':
        raise CliParsingError('The provided CLI command is not an AWS command')
    return tokens
