# Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Common utilities and helpers for the AWS API MCP server."""

from .errors import AwsApiMcpError, Failure
from .helpers import as_json
from loguru import logger
from .models import (
    Context,
    Credentials,
    ProgramValidationRequest,
)

import awslabs.aws_api_mcp_server.core.common.awscli_patch as _awscli_patch

__all__ = [
    'AwsApiMcpError',
    'Failure',
    'as_json',
    'logger',
    'Context',
    'Credentials',
    'ProgramValidationRequest',
]
